@ECHO OFF

SetLocal ENABLEEXTENSIONS

:: PrepMach - AD Migration Utility
:: Gerald W. Gaston - created: 20030611
:: Version: 1.00 [20030611] Created
:: Version: 1.01 [20030618] Updated a few typos. Cosmetic changes only.
:: Version: 1.02 [20030625] Added CSV or TXT option.
:: Version: 1.30 [20060708] Version match with PreMachMigChk

::Usage (All Usage lines MUST start with ::!:):
::!:
::!: PREPMACH v1.3 - AD Migration Utility
::!:
::!: Usage: PREPMACH file_with_PC_names [/CSV|/TXT]
::!:
::!: This script is used to verify that PCs are ready to migrate.
::!: 
::!: The names of PCs that are ready to migrate will be written to a 
::!: file named PREPMACHGOOD.TXT in the current working directory. 
::!: On startup of PREPMACH, if this file already exists you will be
::!: prompted to overwrite or append to it.
::!: If this file doesn't exist after running PREPMACH, no PCs met 
::!: the requirements. 
::!:
::!: An error log file will be created for the PCs that don't meet the 
::!: migration requirements. You will be prompted for the format of the 
::!: error log file if you do not specify /CSV or /TXT as the second 
::!: argument on the command line. The error log file will be named
::!: PREPMACHERR.TXT or PREPMACHERR.CSV depending on your choice.
::!: On startup of PREPMACH, if this file already exists you will be
::!: prompted to overwrite or append to it.
::!: 
::!:
::!: Email any questions/comments to gwgaston@yahoo.com, or STF8B.
::!:

:: Notes:
:: ---------------------
:: This script is for W2K+ CMD only. It will not work with WIN95CMD or 
:: CMD from ReactOS. Due to some extended features, this script will most 
:: likely not run with the current 4NT/TCMD from JP Software without 
:: modification. Yes one could use KIX, PL, VBS, or <insert fav here> 
:: script also for this task. If you want this in another scripting language,
:: please have at it. -GWG
::
:: Dependencies:
:: ---------------------
:: PreMachMigChk.EXE - Program that does the checking...
::                     Expected to be in the CWD that this script is ran in.
::
:: PING.EXE          - MS external utility. Used to verify host is up. 
::                     Expected to be in the path.
::
:: FINDSTR.EXE       - MS external utility. Used for various parsing tasks.	
::
:: CUSRMGR.EXE       - [OPTIONAL] MS W2K resource kit utility. Used to add 
::                     '%TargetDomain%\Domain Admins' to local Administrators group
::                     on the target PC. Both script variables %bAddDomAdmins% and
::                     %TargetDomain% must be defined below for this to happen.
::
:: Windows 2000 is required to run this script.
::
 
::Set DEBUG=YeahBuddy
Set strVersion=1.3
Set panswer=
Set TDSwitch=
Set CDSwitch=
Set WOSwitch=
Set NTSwitch=


:: ***** Customize Section *****
:: *

:: Leave unset if you do not want them used...
Set TargetDomain=ADDOMAIN
Set CurrentDomain=NTRESOURCE

:: Set the following to any value to get them used. Controls the /A and /W switches 
:: passed to PreMigMachChk. 

:: If bWrkstOnly is defined at all then limit to WorkStations only with /W switch.
Set bWrkstOnly=YeahBuddy

:: If bAllowNT4 is defined at all then allow NT4 OS by passing /A switch.
Set bAllowNT4=YeahBuddy

:: If bAddDomAdmins is defined below *AND* TargetDomain is defined above,
:: then "%TargetDomain%\Domain Admins" will be added to the target PCs
:: *IF* PreMachMigChk.EXE returns with exit code 0. Even if you set it here,
:: We unset bAddDomAdmins on down in the script if TargetDomain is not defined.
Set bAddDomAdmins=YeahBuddy

:: *
:: ***** End Customize Section *****



If defined TargetDomain Set TDSwitch= /T %TargetDomain%
If NOT DEFINED TargetDomain Set bAddDomAdmins=
If defined CurrentDomain Set CDSwitch= /C %CurrentDomain%
If defined bWrkstOnly Set WOSwitch= /W
If defined bAllowNT4 Set NTSwitch= /A

:: Resist any urge to put a colon before NOJPCLI
If NOT "%@EVAL[1+1]" == "2" Goto NOJPCLI

Iff Exist @NAME[%0].btm then 
  Call @NAME[%0].btm
  Goto :L8R
Else
  CMD /C "%0" %*
::  @Echo.&Echo.This CMD version of %@UPPER[%@NAME[%0]] will not run under 4NT/TCMD and
::  @Echo.a BTM version of this batch file could not be found. Please 
::  @Echo.locate a BTM version or use CMD.EXE instead. Sorry! 
  Goto :L8R
EndIff

:NOJPCLI

Set logformat=

Set CD=
If defined CD Goto :Arg1Chk
@Echo.
@Echo.ERROR: %~n0 v%strVersion% requires Window 2000 or later.
@Echo.       Please get with the program.
@Echo.
Title PREPMACH v%strVersion%: Aborting - W2K or later required
Goto :L8R

:Arg1Chk
Title Checking Args...
If DEFINED DEBUG @Echo.Checking arguments...
set arg1=%1
If NOT DEFINED arg1 Goto :Syntax
If "A%arg1:~0,1%" == "A?" Goto :Syntax
If "A%arg1:~0,1%" == "A*" Goto :Syntax
If "A%arg1:~0,2%" == "A-?" Goto :Syntax
If "A%arg1:~0,2%" == "A/?" Goto :Syntax
If /I "A%arg1:~0,2%" == "A-h" Goto :Syntax
If /I "A%arg1:~0,2%" == "A/h" Goto :Syntax

Set SvrFile=%1

:Arg2Chk
set arg2=%2
If NOT DEFINED arg2 Goto :LocSupFiles
If "A%arg2:~0,1%" == "A?" Goto :Syntax
If "A%arg2:~0,1%" == "A*" Goto :Syntax
If "A%arg2:~0,2%" == "A-?" Goto :Syntax
If "A%arg2:~0,2%" == "A/?" Goto :Syntax
If /I "A%arg2:~0,2%" == "A-h" Goto :Syntax
If /I "A%arg2:~0,2%" == "A/h" Goto :Syntax
If /I "A%arg2:~0,1%" == "Ac" (Set logformat=CSV&Goto :Arg3Chk)
If /I "A%arg2:~0,2%" == "A/c" (Set logformat=CSV&Goto :Arg3Chk)
If /I "A%arg2:~0,2%" == "A-c" (Set logformat=CSV&Goto :Arg3Chk)
If /I "A%arg2:~0,1%" == "At" (Set logformat=TXT&Goto :Arg3Chk)
If /I "A%arg2:~0,2%" == "A/t" (Set logformat=TXT&Goto :Arg3Chk)
If /I "A%arg2:~0,2%" == "A-t" (Set logformat=TXT&Goto :Arg3Chk)

:Arg3Chk
If NOT "A%3" == "A" goto :Syntax

:LocSupFiles
If exist STOP.IT (
 Del /F /Q STOP.IT > nul 2>&1
)
:: Locate suporting files...
Title Locating support files...
If DEFINED DEBUG @Echo.Locating support files...
Set PMMC=PreMigMachChk.EXE
If NOT exist %PMMC% (Call :MissingFiles PreMachMigChk.EXE & Goto :L8R)
Set PMMC=%PMMC% /M

Set cusrmgr=cusrmgr.EXE
If NOT DEFINED bAddDomAdmins Goto :DefFiles
If NOT exist %cusrmgr% (Call :MissingFiles CUSRMGR.EXE & Goto :L8R)

:DefFiles
Title Defining default file paths and int vars...
If DEFINED DEBUG @Echo.Defining default file paths and int vars...

:SetInitStuff
Set panswer=
Set hostcnt=0
Set goodcnt=0
Set errcnt=0
Set downcnt=0
:: Get pretty date
For /F "tokens=*" %%J in ('date /t') do Set xdate=%%J

If NOT DEFINED SvrFile goto :Syntax
Title Running away with your wife...

:XPchk
:: This check is for XP or 2003 since the output of time /t
:: is different for them
Set runningXP=Y
Set mORnot=

:: This assumes default time settings and calling CMD's int time. 
If DEFINED DEBUG @Echo.Checking LOCAL OS version...
VER | findstr "XP 5.2." > nul 2>&1
If ERRORLEVEL 1 (Set runningXP=N & Set mORnot=m)


If exist %SvrFile% Goto :CSVorTXT
@Echo.&Echo.Can't locate the input file:&Echo.&Echo   %SvrFile%&Echo.
@Echo.Please specify a file that contains a list of PCs.&Echo.
Title PREPMACH v%strVersion%: Can't locate input file
Goto :L8R

:CSVorTXT
If Defined logformat Goto :LogFileDef
Set panswer=
Title ALERT - Input required!
@Echo.&Echo.PREPMACH can create an error log file with a CSV or TXT extension.
@Echo.Choose CSV (comma separated value) if you need to import
@Echo.the error log file into another application (such as Excel). 
@Echo.
Set /p panswer=Do you want the error log file to be in [C]SV or [T]XT format?
If /I "A%panswer:~0,1%" == "AC" (
 @Echo.&Echo.CSV format selected...&Echo.
 Title PREPMACH v%strVersion%: Append mode selected...
 Set logformat=CSV
 Goto :LogFileDef
)
If /I "A%panswer:~0,1%" == "AT" (
 @Echo.&Echo.TXT format selected...&Echo.
 Title PREPMACH v%strVersion%: Append mode selected...
 Set logformat=TXT
 Goto :LogFileDef
)
::Hidden Choice to stop... 
If /I "A%panswer:~0,1%" == "AQ" (
 @Echo.&Echo.Exiting PrepMach...&Echo.
 Title PREPMACH v%strVersion%: Aborting...
 Goto :L8R
)
@Echo.&Echo.INVALID Key selected...
Goto :CSVorTXT

:LogFileDef
Set ERRORLOG=.\PrepMachErr.%logformat%
Set GOODUNSLOG=.\PrepMachGood.TXT


:Errlogchk
Set panswer=
If NOT exist %ERRORLOG% Goto :Goodlogchk
Title ALERT - Input required!
@Echo.&Echo.Error log file exists: %ERRORLOG%
Set /p panswer=Do you want to [A]ppend, [O]verwrite or [Q]uit?
If /I "A%panswer:~0,1%" == "AA" (
 @Echo.&Echo.Append mode selected...&Echo.
 Title PREPMACH v%strVersion%: Append mode selected...
 Goto :Goodlogchk
)
If /I "A%panswer:~0,1%" == "AO" (
 @Echo.&Echo.Removing error log file: %ERRORLOG%...&Echo.
 Del /F %ERRORLOG% > nul 2>&1
 Title PREPMACH v%strVersion%: Removing error log file...
 Goto :Errlogchk
)
If /I "A%panswer:~0,1%" == "AQ" (
 @Echo.&Echo.Exiting PrepMach...&Echo.
 Title PREPMACH v%strVersion%: Aborting - Error log file exits
 Goto :L8R
)
@Echo.&Echo.INVALID Key selected...
Goto :Errlogchk

:Goodlogchk
Set panswer=
If NOT exist %GOODUNSLOG% Goto :DoIT
Title ALERT - Input required!
@Echo.&Echo.The log file of PCs to migrate exists: %GOODUNSLOG%
Set /p panswer=Do you want to [A]ppend, [O]verwrite or [Q]uit?
If /I "A%panswer:~0,1%" == "AA" (
 @Echo.&Echo.Append mode selected...&Echo.
 Title PREPMACH v%strVersion%: Append mode selected...
 Goto :DoIT
)
If /I "A%panswer:~0,1%" == "AO" (
 @Echo.&Echo.Removing migration log file: %GOODUNSLOG%...&Echo.
 Del /F %GOODUNSLOG% > nul 2>&1
 Title PREPMACH v%strVersion%: Removing migration log file...
 Goto :Goodlogchk
)
If /I "A%panswer:~0,1%" == "AQ" (
 @Echo.&Echo.Exiting PrepMach...&Echo.
 Title PREPMACH v%strVersion%: Aborting - Migration log file exits
 Goto :L8R
)
@Echo.&Echo.INVALID Key selected...
Goto :Goodlogchk

:DoIT
@Echo.
Title PREPMACH v%strVersion%: Running
@Echo.PREPMACH Version: %strVersion%
@Echo.
@Echo.Processing file: %SvrFile%
@Echo.
For /F "tokens=*" %%J in ('time /t') do set xtime=%%J%mORnot%
For /F "eol=; delims=, tokens=1" %%S IN (%SvrFile%) do (If NOT Defined STOPITNOW call :ChkHost %%S)
For /F "tokens=*" %%J in ('time /t') do set xtime=%%J%mORnot%
If %hostcnt% GTR 0 (
@Echo.------------------------------------------------------------------------------
)
If NOT Defined STOPITNOW Goto :WrapitUP4um
@Echo.
@Echo.STOP flag file encountered... Shutting down. 
:WrapitUP4um
@Echo.
If %hostcnt% EQU 0 @Echo.%hostcnt% hostnames were processed.
If %hostcnt% EQU 1 @Echo.%hostcnt% hostname was processed.
If %hostcnt% GTR 1 @Echo.%hostcnt% hostnames were processed.
If %downcnt% EQU 1 @Echo.%downcnt% host was offline.
If %downcnt% GTR 1 @Echo.%downcnt% hosts were offline.
If %errcnt% EQU 1 @Echo.%errcnt% host is NOT ready for migration (this includes any hosts offline).
If %errcnt% GTR 1 @Echo.%errcnt% hosts are NOT ready for migration (this includes any hosts offline).
If %goodcnt% EQU 1 @Echo.%goodcnt% host is ready for migration.
If %goodcnt% GTR 1 @Echo.%goodcnt% hosts are ready for migration.
If %goodcnt% GTR 0 (
@Echo.&Echo.One or more PCs are ready for migration. Please see log file:
@Echo.  %GOODUNSLOG%
)
If %errcnt% GTR 0 (
@Echo.&Echo.Problems encountered. Please see error log file:
@Echo.  %ERRORLOG%
)
@Echo.
@Echo.PREPMACH Version: %strVersion% - Finished processing %SvrFile%
Goto :Bye

:ChkHost
If exist STOP.IT (
 Title STOP Flag Encountered: Shutting Down...
 Set STOPITNOW=Y
 Del /F /Q STOP.IT > nul 2>&1
 Goto :EOF
)
@Echo.------------------------------------------------------------------------------
Set targethost=%1

:HootaGamba 
Title Pinging: %targethost%
@Echo.Working on host: %targethost%
@Echo.Started: %xdate%%xtime%
If NOT DEFINED targethost goto :EOF

Set /A hostcnt=%hostcnt%+1

:: Grab the pretty date and time again.
For /F "tokens=*" %%J in ('DATE /t') do Set xdate=%%J
For /F "tokens=*" %%J in ('time /t') do set xtime=%%J%mORnot%


:PingChk
ping -n 2 %targethost% | findstr /LI "ttl=" > nul 2>&1
If %errorlevel% EQU 1 Goto :NOGO
:: Else all is cool and we can...
Title Pinging: %targethost% - UP
@Echo. + Pinged target host... %targethost% is UP.
Goto :Process

:NOGO
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [1] Could not resolve hostname or host offline.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,1,Could not resolve hostname or host offline.>> %ERRORLOG%
)
set /a downcnt=%downcnt%+1
set /a errcnt=%errcnt%+1
Title Pinging: %targethost% - DOWN
@Echo. - Can NOT ping target host... %targethost% is DOWN.
For /F "tokens=*" %%J in ('time /t') do set xtime=%%J%mORnot%
Goto :EOF

:Process

%PMMC% %targethost%%TDSwitch%%CDSwitch%%WOSwitch%%NTSwitch%
set perrlevel=%errorlevel%
If "%perrlevel%" == "0" Goto :AddDA

set /a errcnt=%errcnt%+1

If NOT "A%perrlevel%" == "A1" Goto :Chk2
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [1] Could not resolve hostname or host offline.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,1,Could not resolve hostname or host offline.>> %ERRORLOG%
)
Goto :EndChkHost

:Chk2
set /a errcodechk="%perrlevel% | 2"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :Chk4
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [2] Can not access. Non-NT host or locked down.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,2,Can not access. Non-NT host or locked down>> %ERRORLOG%
)

:Chk4
set /a errcodechk="%perrlevel% | 4"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :Chk8
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [4] NetBIOS name does not match.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,4,NetBIOS name does not match>> %ERRORLOG%
)

:Chk8
set /a errcodechk="%perrlevel% | 8"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :Chk16
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [8] Incorrect domain.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,8,Incorrect domain.>> %ERRORLOG%
)

:Chk16
set /a errcodechk="%perrlevel% | 16"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :Chk32
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [16] Could not access Admin$ share.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,16,Could not access Admin$ share>> %ERRORLOG%
)

:Chk32
set /a errcodechk="%perrlevel% | 32"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :Chk64
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [32] Server detected.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,32,Server detected>> %ERRORLOG%
)

:Chk64
set /a errcodechk="%perrlevel% | 64"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :Chk128
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [64] OS version is less than expected.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,64,OS version is less than expected>> %ERRORLOG%
)

:Chk128
set /a errcodechk="%perrlevel% | 128"
If NOT "A%errcodechk%" == "A%perrlevel%" Goto :EndChkHost
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [128] Could not access remote registry.>> %ERRORLOG%
) Else (
  @Echo.%targethost%,128,Could not access remote registry>> %ERRORLOG%
)
Goto :EndChkHost

:AddDA
If NOT DEFINED bAddDomAdmins Goto :Ready2Mig
%cusrmgr% -u "%TargetDomain%\Domain Admins" -m %targethost% -alg Administrators >nul 2>&1
Set cuerr=%errorlevel%
If "A%cuerr%" == "A1378" (
 @Echo. ! %TargetDomain%\Domain Admins is already in the local Administrators group.
 Goto :Ready2Mig
)
If "A%cuerr%" == "A0" Goto :AddedDASuccess
set /a errcnt=%errcnt%+1
@Echo. - Error adding %TargetDomain%\Domain Admins to the local Administrators group.
If "%logformat%" == "TXT" (
   @Echo.%targethost%: %xdate%%xtime% - [256] Error adding %TargetDomain%\Domain Admins to the local Administrators group.>> %ERRORLOG% 
) Else (
  @Echo.%targethost%,256,Error adding %TargetDomain%\Domain Admins to the local Administrators group>> %ERRORLOG%
)
Goto :EndChkHost

:AddedDASuccess
@Echo. + %TargetDomain%\Domain Admins was added to the local Administrators group.

:Ready2Mig
@Echo.%targethost%>> %GOODUNSLOG%
set /a goodcnt=%goodcnt%+1

:EndChkHost

For /F "tokens=*" %%J in ('time /t') do set xtime=%%J%mORnot%
@Echo.Stopped: %xdate%%xtime%
Goto :EOF

:MissingFiles
(@Echo.&Echo Can not locate a required file: %1)
Title PREPMACH v%strVersion%: Can't locate a required file
Goto :EOF


:Syntax
Title PREPMACH v%strVersion%: Syntax
For /F "tokens=1* delims=:" %%b in ('findstr /BLI "::! " %~f0') Do @Echo.%%c
Goto :L8R

:Bye
Title PREPMACH v%strVersion%: Finished processing %SvrFile%

:L8R
Endlocal
